@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -20,15 +20,16 @@ use structopt::StructOpt;
 use lsp_server::{Connection, Message, Request, RequestId, Response};
 use lsp_types::notification::{DidChangeTextDocument, DidOpenTextDocument, Notification};
 use lsp_types::request::{
-    CodeActionRequest, DocumentColor, DocumentSymbolRequest, ExecuteCommand, GotoDefinition,
+    CodeActionRequest, CodeLensRequest, ColorPresentationRequest, Completion, DocumentColor,
+    DocumentSymbolRequest, ExecuteCommand, GotoDefinition, HoverRequest,
 };
-use lsp_types::request::{ColorPresentationRequest, Completion, HoverRequest};
 use lsp_types::{
-    CodeActionOrCommand, CodeActionProviderCapability, Color, ColorInformation, ColorPresentation,
-    Command, CompletionOptions, DidChangeTextDocumentParams, DidOpenTextDocumentParams,
-    DocumentSymbolResponse, ExecuteCommandOptions, Hover, InitializeParams, Location, OneOf,
-    Position, PublishDiagnosticsParams, Range, ServerCapabilities, SymbolInformation,
-    TextDocumentSyncCapability, Url, WorkDoneProgressOptions,
+    CodeActionOrCommand, CodeActionProviderCapability, CodeLens, CodeLensOptions, Color,
+    ColorInformation, ColorPresentation, Command, CompletionOptions, DidChangeTextDocumentParams,
+    DidOpenTextDocumentParams, DocumentSymbolResponse, ExecuteCommandOptions, Hover,
+    InitializeParams, Location, OneOf, Position, PublishDiagnosticsParams, Range,
+    ServerCapabilities, SymbolInformation, TextDocumentSyncCapability, Url,
+    WorkDoneProgressOptions,
 };
 use sixtyfps_compilerlib::diagnostics::BuildDiagnostics;
 use sixtyfps_compilerlib::langtype::Type;
@@ -50,7 +51,7 @@ struct Cli {
     )]
     include_paths: Vec<std::path::PathBuf>,
 
-    /// The style name for the preview ('native' or 'ugly')
+    /// The style name for the preview ('native', 'fluent' or 'ugly')
     #[structopt(long, name = "style name", default_value)]
     style: String,
 
@@ -148,6 +149,7 @@ fn run_lsp_server() -> Result<(), Error> {
             resolve_provider: None,
             trigger_characters: Some(vec![".".to_owned()]),
             work_done_progress_options: WorkDoneProgressOptions::default(),
+            all_commit_characters: None,
         }),
         definition_provider: Some(OneOf::Left(true)),
         text_document_sync: Some(TextDocumentSyncCapability::Kind(
@@ -160,6 +162,7 @@ fn run_lsp_server() -> Result<(), Error> {
         }),
         document_symbol_provider: Some(OneOf::Left(true)),
         color_provider: Some(true.into()),
+        code_lens_provider: Some(CodeLensOptions { resolve_provider: Some(true) }),
         ..ServerCapabilities::default()
     };
     let server_capabilities = serde_json::to_value(&capabilities).unwrap();
@@ -175,7 +178,7 @@ fn main_loop(connection: &Connection, params: serde_json::Value) -> Result<(), E
     let mut compiler_config =
         CompilerConfiguration::new(sixtyfps_compilerlib::generator::OutputFormat::Interpreter);
     compiler_config.style =
-        Some(if cli_args.style.is_empty() { "ugly".into() } else { cli_args.style });
+        Some(if cli_args.style.is_empty() { "fluent".into() } else { cli_args.style });
     compiler_config.include_paths = cli_args.include_paths;
 
     let mut document_cache = DocumentCache::new(&compiler_config);
@@ -251,11 +254,8 @@ fn handle_request(
             .and_then(|token| get_code_actions(document_cache, token.0.parent()));
         connection.sender.send(Message::Response(Response::new_ok(id, result)))?;
     } else if let Some((id, params)) = cast::<ExecuteCommand>(&mut req) {
-        match params.command.as_str() {
-            SHOW_PREVIEW_COMMAND => {
-                show_preview_command(&params.arguments, connection, document_cache)?
-            }
-            _ => (),
+        if params.command.as_str() == SHOW_PREVIEW_COMMAND {
+            show_preview_command(&params.arguments, connection, document_cache)?
         }
         connection
             .sender
@@ -268,7 +268,7 @@ fn handle_request(
         // representation.
         let requested_color = params.color;
 
-        let color_literal = if requested_color.alpha != 1. {
+        let color_literal = if requested_color.alpha < 1. {
             format!(
                 "#{:0>2x}{:0>2x}{:0>2x}{:0>2x}",
                 (requested_color.red * 255.) as u8,
@@ -290,6 +290,9 @@ fn handle_request(
     } else if let Some((id, params)) = cast::<DocumentSymbolRequest>(&mut req) {
         let result = get_document_symbols(document_cache, &params.text_document);
         connection.sender.send(Message::Response(Response::new_ok(id, result)))?;
+    } else if let Some((id, params)) = cast::<CodeLensRequest>(&mut req) {
+        let result = get_code_lenses(document_cache, &params.text_document);
+        connection.sender.send(Message::Response(Response::new_ok(id, result)))?;
     };
     Ok(())
 }
@@ -357,7 +360,7 @@ fn show_preview_command(
     let component = params.get(1).and_then(|v| v.as_str()).map(|v| v.to_string());
     preview::load_preview(
         connection.sender.clone(),
-        preview::PreviewComponent { path: path_canon.into(), component },
+        preview::PreviewComponent { path: path_canon, component },
         preview::PostLoadBehavior::ShowAfterLoad,
     );
     Ok(())
@@ -415,7 +418,13 @@ fn reload_document(
     let path_canon = dunce::canonicalize(&path).unwrap_or_else(|_| path.to_owned());
     preview::set_contents(&path_canon, content.clone());
     let mut diag = BuildDiagnostics::default();
-    spin_on::spin_on(document_cache.documents.load_file(&path_canon, &path, content, &mut diag));
+    spin_on::spin_on(document_cache.documents.load_file(
+        &path_canon,
+        &path,
+        content,
+        false,
+        &mut diag,
+    ));
 
     // Always provide diagnostics for all files. Empty diagnostics clear any previous ones.
     let mut lsp_diags: HashMap<Url, Vec<lsp_types::Diagnostic>> = core::iter::once(&path)
@@ -488,14 +497,14 @@ fn get_code_actions(
         .or_else(|| {
             syntax_nodes::DeclaredIdentifier::new(node.clone())
                 .and_then(|n| n.parent())
-                .and_then(|p| syntax_nodes::Component::new(p))
+                .and_then(syntax_nodes::Component::new)
         })
         .or_else(|| {
             syntax_nodes::QualifiedName::new(node.clone())
                 .and_then(|n| n.parent())
-                .and_then(|p| syntax_nodes::Element::new(p))
+                .and_then(syntax_nodes::Element::new)
                 .and_then(|n| n.parent())
-                .and_then(|p| syntax_nodes::Component::new(p))
+                .and_then(syntax_nodes::Component::new)
         })?;
 
     let component_name =
@@ -525,8 +534,8 @@ fn get_document_color(
                 let shift = |s: u32| -> f32 { ((col >> s) & 0xff) as f32 / 255. };
                 result.push(ColorInformation {
                     range: Range::new(
-                        document_cache.byte_offset_to_position(range.start().into(), &uri)?,
-                        document_cache.byte_offset_to_position(range.end().into(), &uri)?,
+                        document_cache.byte_offset_to_position(range.start().into(), uri)?,
+                        document_cache.byte_offset_to_position(range.end().into(), uri)?,
                     ),
                     color: Color {
                         alpha: shift(24),
@@ -560,12 +569,11 @@ fn get_document_symbols(
 
     let inner_components = doc.inner_components.clone();
     let inner_structs = doc.inner_structs.clone();
-    drop(doc);
     let mut make_range = |node: &SyntaxNode| {
         let r = node.text_range();
         Some(Range::new(
-            document_cache.byte_offset_to_position(r.start().into(), &uri)?,
-            document_cache.byte_offset_to_position(r.end().into(), &uri)?,
+            document_cache.byte_offset_to_position(r.start().into(), uri)?,
+            document_cache.byte_offset_to_position(r.end().into(), uri)?,
         ))
     };
 
@@ -598,3 +606,38 @@ fn get_document_symbols(
 
     // TODO: add the structs
 }
+
+fn get_code_lenses(
+    document_cache: &mut DocumentCache,
+    text_document: &lsp_types::TextDocumentIdentifier,
+) -> Option<Vec<CodeLens>> {
+    let uri = &text_document.uri;
+    let filepath = uri.to_file_path().ok()?;
+    let doc = document_cache.documents.get_document(&filepath)?;
+
+    let inner_components = doc.inner_components.clone();
+    let mut make_range = |node: &SyntaxNode| {
+        let r = node.text_range();
+        Some(Range::new(
+            document_cache.byte_offset_to_position(r.start().into(), uri)?,
+            document_cache.byte_offset_to_position(r.end().into(), uri)?,
+        ))
+    };
+
+    let r = inner_components
+        .iter()
+        .filter(|c| !c.is_global())
+        .filter_map(|c| {
+            Some(CodeLens {
+                range: make_range(c.root_element.borrow().node.as_ref()?)?,
+                command: Some(Command::new(
+                    "▶ Show preview".into(),
+                    SHOW_PREVIEW_COMMAND.into(),
+                    Some(vec![filepath.to_str()?.into(), c.id.as_str().into()]),
+                )),
+                data: None,
+            })
+        })
+        .collect::<Vec<_>>();
+    Some(r)
+}
