@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -35,7 +35,7 @@ enum RequestedGuiEventLoopState {
     QuitLoop,
 }
 
-static GUI_EVENT_LOOP_NOTIFIER: Lazy<Condvar> = Lazy::new(|| Condvar::new());
+static GUI_EVENT_LOOP_NOTIFIER: Lazy<Condvar> = Lazy::new(Condvar::new);
 static GUI_EVENT_LOOP_STATE_REQUEST: Lazy<Mutex<RequestedGuiEventLoopState>> =
     Lazy::new(|| Mutex::new(RequestedGuiEventLoopState::Uninitialized));
 
@@ -140,7 +140,7 @@ pub fn load_preview(
     post_load_behavior: PostLoadBehavior,
 ) {
     use std::sync::atomic::{AtomicU32, Ordering};
-    const PENDING_EVENTS: AtomicU32 = AtomicU32::new(0);
+    static PENDING_EVENTS: AtomicU32 = AtomicU32::new(0);
     if PENDING_EVENTS.load(Ordering::SeqCst) > 0 {
         return;
     }
@@ -188,7 +188,7 @@ pub fn set_contents(path: &Path, content: String) {
 /// In any was, register it as a dependency
 fn get_file_from_cache(path: PathBuf) -> Option<String> {
     let mut cache = CONTENT_CACHE.get_or_init(Default::default).lock().unwrap();
-    let r = cache.source_code.get(&path).map(|r| r.clone());
+    let r = cache.source_code.get(&path).cloned();
     cache.dependency.insert(path);
     r
 }
@@ -206,7 +206,7 @@ async fn reload_preview(
         cache.current = preview_component.clone();
     }
 
-    let mut builder = sixtyfps_interpreter::ComponentCompiler::new();
+    let mut builder = sixtyfps_interpreter::ComponentCompiler::default();
     let cli_args = super::Cli::from_args();
     if !cli_args.style.is_empty() {
         builder.set_style(cli_args.style)
@@ -269,7 +269,7 @@ fn notify_diagnostics(
             continue;
         }
         let uri = lsp_types::Url::from_file_path(d.source_file().unwrap()).unwrap();
-        lsp_diags.entry(uri).or_default().push(crate::util::to_lsp_diag(&d));
+        lsp_diags.entry(uri).or_default().push(crate::util::to_lsp_diag(d));
     }
 
     for (uri, diagnostics) in lsp_diags {
